@@ -13,7 +13,7 @@
 #   limitations under the License.
 
 from prometheus_client import Counter
-from th2_grpc_common.common_pb2 import MessageBatch
+from th2_grpc_common.common_pb2 import MessageBatch, MessageGroupBatch, MessageGroup, AnyMessage
 
 from th2_common.schema.message.impl.rabbitmq.abstract_rabbit_sender import AbstractRabbitSender
 
@@ -36,5 +36,9 @@ class RabbitParsedBatchSender(AbstractRabbitSender):
     def get_messages(self, batch: MessageBatch) -> list:
         return batch.messages
 
-    def value_to_bytes(self, value: MessageBatch):
-        return value.SerializeToString()
+    @staticmethod
+    def value_to_bytes(batch: MessageBatch):
+        messages = [AnyMessage(message=msg) for msg in batch.messages]
+        group = MessageGroup(messages=messages)
+        group_batch = MessageGroupBatch(groups=[group])
+        return group_batch.SerializeToString()
